﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace ELTE.Basics.CoffeeShop
{
    /// <summary>
    /// Menü osztálya.
    /// </summary>
    class Menu
    {
        private List<Order> _Orders;

        /// <summary>
        /// Menü létrehozása. 
        /// </summary>
        public Menu()
        {
            _Orders = new List<Order>();
        }
        /// <summary>
        /// Menü futtatása.
        /// </summary>
        public void Run()
        {
            String choice;
            Int32 nr;

            do
            {
                WriteMainMenu();
                choice = Console.ReadLine();
                switch (choice)
                {
                    case "1":
                        AddOrder();
                        break;
                    case "2":
                        WriteRatingsMenu();
                        choice = Console.ReadLine();
                        switch (choice)
                        {
                            case "1":
                                ShowRatings();
                                break;
                            case "2":
                                do
                                {
                                    Console.Write("Rendelés azonosítója: ");
                                }
                                while (!Int32.TryParse(Console.ReadLine(), out nr)); // addig kérjük be, amíg számot nem üt a felhasználó
                                ShowRatingsForOrder(nr);
                                break;
                            case "3":
                                do
                                {
                                    Console.Write("Törzsvásárlói kártya száma: ");
                                }
                                while (!Int32.TryParse(Console.ReadLine(), out nr));
                                ShowRatingsForCard(nr);
                                break;
                        }
                        break;
                }
            }
            while (choice != "X" && choice != "x");

        }
        /// <summary>
        /// Adatok betöltése.
        /// </summary>
        public void LoadData() 
        {
            try
            {
                StreamReader reader = new StreamReader("data.dat");
                String row, name;
                Int32 orderNumber, cardNumber, itemCount, cost;

                if (!reader.EndOfStream)
                {
                    row = reader.ReadLine();
                    while (!reader.EndOfStream)
                    {
                        String[] rowArray = row.Split(';'); // a sort felbontjuk a ; mentén
                        orderNumber = Int32.Parse(rowArray[0]); // a sor elemeit átalakítjuk a megfelelő típusra
                        itemCount = Int32.Parse(rowArray[1]);
                        cardNumber = Int32.Parse(rowArray[2]);

                        _Orders.Add(new Order(orderNumber, cardNumber));
                        for (int i = 0; i < itemCount && !reader.EndOfStream; i++)
                        {
                            rowArray = reader.ReadLine().Split(';');
                            switch (rowArray[0])
                            {
                                case "h":
                                    name = rowArray[1]; // étel esetén feldolgozzuk a sor többi elemét is
                                    cost = Int32.Parse(rowArray[2]);
                                    _Orders[_Orders.Count - 1].AddItem(new Hamburger(name, cost)); // a típusnak megfelelő elemet vesszük fel
                                    break;
                                case "u":
                                    name = rowArray[1];
                                    cost = Int32.Parse(rowArray[2]);
                                    _Orders[_Orders.Count - 1].AddItem(new Ufo(name, cost));
                                    break;
                                case "p":
                                    name = rowArray[1];
                                    cost = Int32.Parse(rowArray[2]);
                                    _Orders[_Orders.Count - 1].AddItem(new Pancake(name, cost));
                                    break;
                                case "t":
                                    _Orders[_Orders.Count - 1].AddItem(new Tea());
                                    break;
                                case "n":
                                    _Orders[_Orders.Count - 1].AddItem(new Orange());
                                    break;
                                case "k":
                                    _Orders[_Orders.Count - 1].AddItem(new Coke());
                                    break;
                            }
                        }
                        row = reader.ReadLine();
                    }
                    reader.Close();
                }
            }
            catch (Exception)
            {
                Console.WriteLine("Az adatok betöltése sikertelen!");
            }
        }
        /// <summary>
        /// Adatok mentése.
        /// </summary>
        public void SaveData()
        {
            try
            {
                StreamWriter writer = new StreamWriter("data.dat"); // a data.dat fájlba írunk, felülírjuk az eddigi tartalmat
                for (Int32 i = 0; i < _Orders.Count; i++) // végigmegyünk a rendeléseken
                {
                    writer.WriteLine(_Orders[i].Number.ToString() + ";" + _Orders[i].ItemCount.ToString() + ";" + _Orders[i].CardNumber.ToString());
                    for (Int32 j = 0; j < _Orders[i].ItemCount; j++) // végigmegyünk a rendelés összes elemén
                    {
                        writer.WriteLine(_Orders[i][j].Type + ";" + _Orders[i][j].Name + ";" + _Orders[i][j].GrossValue.ToString());
                    }
                }
                writer.Close();
            }
            catch (Exception)
            {
                Console.WriteLine("Az adatok mentése sikertelen!");
            }
        }
        /// <summary>
        /// Főmenü kiírása.
        /// </summary>
        private void WriteMainMenu() 
        {
            Console.WriteLine();
            Console.WriteLine("Marika néni kávézója");
            Console.WriteLine("====================");
            Console.WriteLine();
            Console.WriteLine("Kérem válasszon:");
            Console.WriteLine("  1: rendelés felvétele");
            Console.WriteLine("  2: rendelések lekérdezése");
            Console.WriteLine("  X: kilépés");
            Console.WriteLine("Választás: ");
        }
        /// <summary>
        /// Lekérdezések menü kiírása.
        /// </summary>
        private void WriteRatingsMenu() 
        {
            Console.WriteLine();
            Console.WriteLine("Rendelések lekérdezése");
            Console.WriteLine("======================");
            Console.WriteLine();
            Console.WriteLine("Kérem válasszon:");
            Console.WriteLine("  1: összes");
            Console.WriteLine("  2: adott rendelésre");
            Console.WriteLine("  3: adott törzsvásárlóra");
            Console.WriteLine("  X: kilépés");
            Console.WriteLine("Választás: ");
        }
        /// <summary>
        /// Új reldelés felvétele.
        /// </summary>
        private void AddOrder() 
        {
            Int32 orderNumber = _Orders.Count + 1;
            Int32 cardNumber, cost;
            String choice;
            String name;

            do
            {
                Console.Write("Törzsvásárlói kártya száma (0, ha nincs): ");
            }
            while (!Int32.TryParse(Console.ReadLine(), out cardNumber)); // ciklussal kérünk be, amíg számot nem ad a felhasználó

            _Orders.Add(new Order(orderNumber, cardNumber));

            do
            {
                Console.WriteLine();
                Console.WriteLine("1: Hamburger hozzáadása");
                Console.WriteLine("2: Ufo hozzáadása");
                Console.WriteLine("3: palacsinta hozzáadása");
                Console.WriteLine("4: tea hozzáadása");
                Console.WriteLine("5: narancslé hozzáadása");
                Console.WriteLine("6: kóla hozzáadása");
                Console.WriteLine("X: főmenü");
                Console.WriteLine("Választás: ");

                choice = Console.ReadLine();
                switch (choice)
                {
                    case "1":
                        Console.Write("Név: ");
                        name = Console.ReadLine();
                        do
                        {
                            Console.Write("Ár: ");
                        }
                        while (!Int32.TryParse(Console.ReadLine(), out cost));

                        _Orders[_Orders.Count - 1].AddItem(new Hamburger(name, cost));
                        break;
                    case "2":
                        Console.Write("Név: ");
                        name = Console.ReadLine();
                        do
                        {
                            Console.Write("Ár: ");
                        }
                        while (!Int32.TryParse(Console.ReadLine(), out cost));

                        _Orders[_Orders.Count - 1].AddItem(new Ufo(name, cost));
                        break;
                    case "3":
                        Console.Write("Név: ");
                        name = Console.ReadLine();
                        do
                        {
                            Console.Write("Ár: ");
                        }
                        while (!Int32.TryParse(Console.ReadLine(), out cost));

                        _Orders[_Orders.Count - 1].AddItem(new Pancake(name, cost));
                        break;
                    case "4":
                        _Orders[_Orders.Count - 1].AddItem(new Tea());
                        break;
                    case "5":
                        _Orders[_Orders.Count - 1].AddItem(new Orange());
                        break;
                    case "6":
                        _Orders[_Orders.Count - 1].AddItem(new Coke());
                        break;
                }
            } while (choice != "X" && choice != "x");
        }
        /// <summary>
        /// Adott rendelés lekérdezése.
        /// </summary>
        /// <param name="orderNumber">A rendelés száma.</param>
        private void ShowRatingsForOrder(Int32 orderNumber) 
        {
            if (orderNumber > 0 && orderNumber <= _Orders.Count) // ha jó sorszámot adott meg
            {
                Console.WriteLine("Nettó érték: " + _Orders[orderNumber - 1].NetValue); // visszaadjuk az adott rendelés nettó-bruttó értékét
                Console.WriteLine("Bruttó érték: " + _Orders[orderNumber - 1].GrossValue);
            }
            else
            {
                Console.WriteLine("A rendelés sorszámának 1 és " + _Orders.Count + " között kell lennie!");
            }
        }
        /// <summary>
        /// Adott törzsvásárló rendeléseinek lekérdezése.
        /// </summary>
        /// <param name="cardNumber"></param>
        private void ShowRatingsForCard(Int32 cardNumber) 
        {
            Int32 netValue = 0, grossValue = 0;
            for (Int32 i = 0; i < _Orders.Count; i++)
            {
                if (_Orders[i].CardNumber == cardNumber) // az összegzés feltétele az azonos kártyaszám
                {
                    netValue += _Orders[i].NetValue;
                    grossValue += _Orders[i].GrossValue;
                }
            }
            Console.WriteLine("Nettó érték: " + netValue);
            Console.WriteLine("Bruttó érték: " + grossValue);
        }
        /// <summary>
        /// Összes rendelés lekérdezése.
        /// </summary>
        private void ShowRatings() 
        {
            Int32 netValue = 0, grossValue = 0;
            for (Int32 i = 0; i < _Orders.Count; i++)
            {
                netValue += _Orders[i].NetValue;
                grossValue += _Orders[i].GrossValue;
            }
            Console.WriteLine("Nettó érték: " + netValue);
            Console.WriteLine("Bruttó érték: " + grossValue);
        }
    }
}
